import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.Timer;

import com.phidgets.AdvancedServoPhidget;
import com.phidgets.PhidgetException;
import com.phidgets.RFIDPhidget;
import com.phidgets.event.AttachEvent;
import com.phidgets.event.AttachListener;
import com.phidgets.event.CurrentChangeEvent;
import com.phidgets.event.CurrentChangeListener;
import com.phidgets.event.DetachEvent;
import com.phidgets.event.DetachListener;
import com.phidgets.event.ServerConnectEvent;
import com.phidgets.event.ServerConnectListener;
import com.phidgets.event.ServerDisconnectEvent;
import com.phidgets.event.ServerDisconnectListener;

public class Controller {
	View view = new View();
	Model model = new Model();

	private AdvancedServoPhidget ifk;
	private RFIDPhidget ifp;
	private boolean trigger_not_active = true;
	Timer timer;

	public Controller() {
		try {
			ifk = new AdvancedServoPhidget();
			ifp = new RFIDPhidget();
		} catch (PhidgetException ex) {
			System.out.println(ex.getDescription()+ " Phidget Error" + ex.getErrorNumber()+" "+JOptionPane.ERROR_MESSAGE);
		}

		/* Controls */
		/* MouseListener */
		//view.addListenerToBaseLeft(new baseLeftListener());
		view.addMouseListenerToBaseLeft(new mouseListener("baseLeft"));
		//view.addListenerToBaseRight(new baseRightListener());
		view.addMouseListenerToBaseRight(new mouseListener("baseRight"));
		//view.addListenerToArmForward(new armForwardListener());
		view.addMouseListenerToArmForward(new mouseListener("armForward"));
		//view.addListenerToArmBackward(new armBackwardListener());
		view.addMouseListenerToArmBackward(new mouseListener("armBackward"));
		//view.addListenerToHeadUp(new headUpListener());
		view.addMouseListenerToHeadUp(new mouseListener("headUp"));
		//view.addListenerToHeadDown(new headDownListener());
		view.addMouseListenerToHeadDown(new mouseListener("headDown"));
		//view.addMouseListenerToTrigger(new mouseListener("trigger"));
		view.addMouseListenerToTrigger(new mouseListener("trigger"));
		//view.addListenerToTrigger(new triggerListener());
		view.addListenerToKey(new keyListener());
		/* KeyListener */
		view.addListenerToConnect(new connectListener());
		view.addListenerToPump(new pumpListener());
		view.addListenerToEngage(new engageListener());
		view.addListenerToReset(new resetListener());

		/* File menu */
		view.addListenerToConnectTo(new connectListener());
		view.addListenerToQuit(new quitListener());
		
		/* Controls menu */
		view.addListenerToControlsReset(new resetListener());
		view.addListenerToControlsEngage(new engageListener());
		view.addListenerToControlsBaseLeft(new baseLeftListener());
		view.addListenerToControlsBaseRight(new baseRightListener());
		view.addListenerToControlsHeadUp(new headUpListener());
		view.addListenerToControlsHeadDown(new headDownListener());
		view.addListenerToControlsArmForward(new armForwardListener());
		view.addListenerToControlsArmBackward(new armBackwardListener());
		view.addListenerToControlsPump(new pumpListener());
		//view.addListenerToControlsTrigger(new triggerListener());
		
		/* About menu */
		view.addListenerToWiki(new wikiListener());
		
		/* Phidget */
		ifk.addServerConnectListener(new serverconnectListener());
		ifk.addServerDisconnectListener(new serverdisconnectListener());
		ifk.addAttachListener(new phidgetattachListener());
		ifk.addDetachListener(new phidgetdetachListener());
		ifk.addCurrentChangeListener(new currentChangeListener());
		ifp.addServerConnectListener(new serverconnectListener());
		ifp.addServerDisconnectListener(new serverdisconnectListener());
		ifp.addAttachListener(new phidgetattachListener());
		ifp.addDetachListener(new phidgetdetachListener());
	}

	/**
	 * Reset servos to default position, velocity, acceleration and max/min posisiton.
	 * 
	 * @return
	 */
	public void resetServos () {
		try {
			System.out.println("Reseting servo configuration...");
			ifk.setAcceleration(model.getArmservo(), model.getArmpos_acc());
			ifk.setVelocityLimit(model.getArmservo(), model.getArmpos_vel());
			ifk.setPosition(model.getArmservo(), model.getArmpos_def());
			ifk.setPositionMin(model.getArmservo(), model.getArmpos_min());
			ifk.setPositionMax(model.getArmservo(), model.getArmpos_max());
			view.setStatusArmpos(Double.toString(model.getArmpos_def()),  Double.toString(model.getArmpos_current()));		
			model.setArmpos(model.getArmpos_def());
			ifk.setSpeedRampingOn(model.getArmservo(), true);
			ifk.setEngaged(model.getArmservo(), true);

			ifk.setAcceleration(model.getHeadservo(), model.getHeadpos_acc());
			ifk.setVelocityLimit(model.getHeadservo(), model.getHeadpos_vel());
			ifk.setPosition(model.getHeadservo(), model.getHeadpos_def());
			ifk.setPositionMin(model.getHeadservo(), model.getHeadpos_min());
			ifk.setPositionMax(model.getHeadservo(), model.getHeadpos_max());
			view.setHeadpos(Double.toString(model.getHeadpos_def()), Double.toString(model.getHeadpos_current()));
			model.setHeadpos(model.getHeadpos_def());
			ifk.setSpeedRampingOn(model.getHeadservo(), true);
			ifk.setEngaged(model.getHeadservo(), true);
			/*
			ifk.setAcceleration(model.getBaseservo(), model.getBasepos_acc());
			 */
			ifk.setVelocityLimit(model.getBaseservo(), model.getBasepos_vel());
			ifk.setPosition(model.getBaseservo(), model.getBasepos_def());
			ifk.setPositionMin(model.getBaseservo(), model.getBasepos_min());
			ifk.setPositionMax(model.getBaseservo(), model.getBasepos_max());
			view.setStatusBasepos(Double.toString(model.getBasepos_def()),  Double.toString(model.getBasepos_current()));
			model.setBasepos(model.getBasepos_def());
			ifk.setSpeedRampingOn(model.getBaseservo(), true);
			ifk.setEngaged(model.getBaseservo(), true);

			ifp.setOutputState(model.getTriggeroutput(), model.isTrigger_def());
			view.setStatusTrigger(Boolean.toString(model.isTrigger_def()));

			model.setEngage(model.isTrigger_def());
			view.setControlsEngage(model.isEngage());		

		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Open a webpage in the default webbrowser. Currently only Windows and Mac OS X is supported.
	 * 
	 * @param uri link to the webpage to show
	 */
	public void openWebpage(String uri) {
		String osName= System.getProperty("os.name");
		try {
			if (osName.equals("Mac OS X")) {
				Runtime.getRuntime().exec("open "+uri);
			} else if (osName.contains("Windows")) {				
				Runtime.getRuntime().exec("cmd /C start "+uri);
			} else {
				System.out.println("Cannot open website!");
			}
		} catch (IOException e) {}
	}


	/**
	 * Connect to server.
	 */
	public void connect() {

		try {
			// Close any open threads, hacky I know
			ifp.close();
			ifk.close();

			//System.out.println("Connected: " + model.isConnected());
			if (model.isConnected()) {
				System.out.println("Disconnecting from current server.");
				view.setConnect_button("  Connect  ");
				model.setConnected(false);
				view.setStatusConnected(Boolean.toString(model.isConnected()));
				view.setStatusConnectedColor(new Color(200,0,0));
			} else {
				model.setAddress(view.getAddress());
				System.out.println("Connecting to server at "+model.getAddress());
				ifk.open(84174, model.getAddress(), 5001, model.getServer_pass());
				ifp.open(77985, model.getAddress(), 5001, model.getServer_pass());	
			}
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Connected to server.
	 */
	public void connected() {

		try {
			model.setConnected(true);
			view.setStatusConnected(Boolean.toString(model.isConnected()));
			view.setStatusConnectedColor(new Color(0,200,0));
			view.setConnect_button("Disconnect");
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Disconnected to server.
	 */
	public void disconnected() {

		// NOT WORKING when pressing "Disconnected", moved same code below, to connect()
		try {
			model.setConnected(false);
			view.setConnect_button("  Connect  ");
			view.setStatusConnected(Boolean.toString(model.isConnected()));
			view.setStatusConnectedColor(new Color(200,0,0));
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Move head up.
	 */
	public void headUp() {
		System.out.println("head min: "+model.getHeadpos_min()+" head min absolute: "+model.getHeadpos_min_absolute());

		try {

			double newpos = model.getHeadpos()-model.getHeadpos_res();
			if (newpos <= model.getHeadpos_min()) {
				newpos = model.getHeadpos_min();
				System.out.println("Min position reached!");	
			}
			ifk.setPosition(model.getHeadservo(), newpos);
			model.setHeadpos(newpos);
			view.setHeadpos(Double.toString(newpos), Double.toString(model.getHeadpos_current()));
			System.out.println("head up min: "+model.getHeadpos_min()+" head up pos: "+newpos);			
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/** 
	 * Move head down.
	 */
	public void headDown() {
		try {
			double newpos = model.getHeadpos()+model.getHeadpos_res();
			if (newpos >= model.getHeadpos_max()) {
				newpos = model.getHeadpos_max();
				System.out.println("Max position reached!");	
			}
			ifk.setPosition(model.getHeadservo(), newpos);
			model.setHeadpos(newpos);
			view.setHeadpos(Double.toString(newpos), Double.toString(model.getHeadpos_current()));
			System.out.println("head down max: "+model.getHeadpos_max()+" head down pos: "+newpos);			
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/** 
	 * Move arm forward.
	 */
	public void armForward() {

		try {
			// calc new arm pos
			double newpos = model.getArmpos()-model.getArmpos_res();

			// calc new head pos
			double newheadpos = model.getHeadpos()-model.getArmpos_res();

			// check if new pos is not outside min arm pos
			if (newpos <= model.getArmpos_min()) {
				newpos = model.getArmpos_min();
				newheadpos = model.getHeadpos()-(model.getArmpos()-model.getArmpos_min());
				System.out.println("Min position reached!");	
			}
			
			/* ARM */
			// set and execute new pos
			ifk.setPosition(model.getArmservo(), newpos);
			model.setArmpos(newpos);
			view.setStatusArmpos(Double.toString(newpos), Double.toString(model.getArmpos_current()));

			// new parallel head pos - defliction from 90degrees, standard pos
			double defliction;
			double newdef;
			if (model.getArmpos() < model.getArmpos_def()) {
				defliction = model.getArmpos_def() - model.getArmpos();
				newdef = model.getArmpos_def()+defliction;
			} else {
				defliction = model.getArmpos() - model.getArmpos_def();
				newdef = model.getArmpos_def()-defliction;
			}
			System.out.println("arm pos def: "+model.getArmpos_def()+" arm pos: "+model.getArmpos());
			System.out.println("head pos min: "+model.getHeadpos_min()+" arm deflict: "+defliction);

			/* HEAD */
			// set new min pos for head
			// min pos 0
			double newheadmin = model.getHeadpos_min()-defliction;
			if (newheadmin >= model.getHeadpos_min_absolute()) {
				model.setHeadpos_min(newheadmin);
				ifk.setPositionMin(model.getHeadservo(), newheadmin);
			} else {
				model.setHeadpos_min(model.getHeadpos_min_absolute());
				ifk.setPositionMin(model.getHeadservo(), model.getHeadpos_min_absolute());
			}
			System.out.println("New head min: "+model.getHeadpos_min());

			/*
			double newheadmin = newdef-model.getHeadpos_range_ang();
			
			if (newheadmin >= model.getHeadpos_min_absolute()) {
				model.setHeadpos_min(newheadmin);
				ifk.setPositionMin(model.getHeadservo(), newheadmin);
			} else {
				model.setHeadpos_min(model.getHeadpos_min_absolute());
				ifk.setPositionMin(model.getHeadservo(), model.getHeadpos_min_absolute());
			}
			System.out.println("New head min: "+model.getHeadpos_min());

			

			double newheadmax = newdef+model.getHeadpos_range_ang();
			
			if (newheadmax <= model.getHeadpos_max_absolute()) {
				model.setHeadpos_max(newheadmax);
				ifk.setPositionMax(model.getHeadservo(), newheadmax);
			} else {
				model.setHeadpos_max(model.getHeadpos_max_absolute());				
				ifk.setPositionMax(model.getHeadservo(), model.getHeadpos_max_absolute());
			}
			System.out.println("New head max: "+model.getHeadpos_max());
			*/
			
			// set parallel head pos
			if (newheadpos >= model.getHeadpos_min()) {
				ifk.setPosition(model.getHeadservo(),newheadpos);
				model.setHeadpos(newheadpos);
				view.setHeadpos(Double.toString(newheadpos), Double.toString(model.getHeadpos_current()));
			}

		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Move arm backward.
	 */
	public void armBackward() {

		try {
			// calc new arm pos
			double newpos = model.getArmpos()+model.getArmpos_res();

			// calc new head pos
			double newheadpos = model.getHeadpos()+model.getArmpos_res();

			// check if new arm pos is not outside max pos arm
			if (newpos >= model.getArmpos_max()) {
				newpos = model.getArmpos_max();
				newheadpos = model.getHeadpos()+(model.getArmpos_max()-model.getArmpos());
				System.out.println("Max position reached!");	
			}
			
			/* ARM */
			// set and execute new arm pos
			ifk.setPosition(model.getArmservo(), newpos);
			model.setArmpos(newpos);
			view.setStatusArmpos(Double.toString(newpos), Double.toString(model.getArmpos_current()));

			// new parallel head pos - defliction from 90degrees, standard pos
			double defliction;
			double newdef;
			if (model.getArmpos() < model.getArmpos_def()) {
				defliction = model.getArmpos_def() - model.getArmpos();
				newdef = model.getArmpos_def()+defliction;
			} else {
				defliction = model.getArmpos() - model.getArmpos_def();
				newdef = model.getArmpos_def()-defliction;
			}
			System.out.println("arm pos def: "+model.getArmpos_def()+" arm pos: "+model.getArmpos());
			System.out.println("head pos max: "+model.getHeadpos_max()+" arm deflict: "+defliction);

			/* HEAD */
			// set new max pos for head
			// max pos 255, head move down, when pulling backward
			double newheadmax = model.getHeadpos_max()+defliction;
			if (newheadmax <= model.getHeadpos_max_absolute()) {
				model.setHeadpos_max(newheadmax);
				ifk.setPositionMax(model.getHeadservo(), newheadmax);
			} else {
				model.setHeadpos_max(model.getHeadpos_max_absolute());				
				ifk.setPositionMax(model.getHeadservo(), model.getHeadpos_max_absolute());
			}
			System.out.println("New head max: "+model.getHeadpos_max());

			/*
			double newheadmax = newdef+model.getHeadpos_range_ang();
			
			if (newheadmax <= model.getHeadpos_max_absolute()) {
				model.setHeadpos_max(newheadmax);
				ifk.setPositionMax(model.getHeadservo(), newheadmax);
			} else {
				model.setHeadpos_max(model.getHeadpos_max_absolute());				
				ifk.setPositionMax(model.getHeadservo(), model.getHeadpos_max_absolute());
			}
			System.out.println("New head max: "+model.getHeadpos_max());
			

			double newheadmin = newdef-model.getHeadpos_range_ang();
			
			if (newheadmin >= model.getHeadpos_min_absolute()) {
				model.setHeadpos_min(newheadmin);
				ifk.setPositionMin(model.getHeadservo(), newheadmin);
			} else {
				model.setHeadpos_min(model.getHeadpos_min_absolute());
				ifk.setPositionMin(model.getHeadservo(), model.getHeadpos_min_absolute());
			}
			System.out.println("New head min: "+model.getHeadpos_min());
			*/
			
			// set parallel head pos
			if (newheadpos <= model.getHeadpos_max()) {
				ifk.setPosition(model.getHeadservo(),newheadpos);
				model.setHeadpos(newheadpos);
				view.setHeadpos(Double.toString(newheadpos), Double.toString(model.getArmpos_current()));
			}

		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Move base left.
	 */
	public void baseLeft() {

		try {
			double newpos = model.getBasepos()-model.getBasepos_res();
			if (newpos <= model.getBasepos_min()) {
				newpos = model.getBasepos_min();
				System.out.println("Min position reached!");	
			}
			ifk.setPosition(model.getBaseservo(), newpos);
			model.setBasepos(newpos);
			view.setStatusBasepos(Double.toString(newpos), Double.toString(model.getBasepos_current()));
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/** Move base right.
	 * 
	 */
	public void baseRight() {

		try {
			double newpos = model.getBasepos()+model.getBasepos_res();
			if (newpos >= model.getBasepos_max()) {					
				newpos = model.getBasepos_max();
				System.out.println("Max position reached!");	
			}
			ifk.setPosition(model.getBaseservo(), newpos);
			model.setBasepos(newpos);
			view.setStatusBasepos(Double.toString(newpos), Double.toString(model.getBasepos_current()));
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Toggle pump.
	 */
	public void pump() {

		try {
			boolean pump_state = ifp.getOutputState(model.getPumpoutput());
			ifp.setOutputState(model.getPumpoutput(), !pump_state);
			model.setPump(!model.isPump());
			view.setStatusPump(Boolean.toString(model.isPump()));

			String pump_txt = (pump_state ? "Pump On" : "Pump Off");
			view.setControlsPump(pump_txt);
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	/**
	 * Toggle trigger.
	 */
	public void trigger() {

		try {
			boolean trigger_state = ifp.getOutputState(model.getTriggeroutput());
			ifp.setOutputState(model.getTriggeroutput(), !trigger_state);
			model.setTrigger(!model.isTrigger());
			view.setStatusTrigger(Boolean.toString(model.isTrigger()));
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}


	/**
	 * Toggle engage bit.
	 */
	public void engage() {

		try {
			boolean engage_state = ifk.getEngaged(model.getArmservo());
			ifk.setEngaged(model.getArmservo(), !engage_state);
			ifk.setEngaged(model.getBaseservo(), !engage_state);
			ifk.setEngaged(model.getHeadservo(), !engage_state);
			model.setEngage(!model.isEngage());
			System.out.println("Setting engage state to: "+!engage_state);
			view.setControlsEngage(model.isEngage());
		} catch (Exception ex) {
			System.out.println("Error: "+ex);
		}
	}

	// LISTENERS
	class currentChangeListener implements CurrentChangeListener{
		public void currentChanged(CurrentChangeEvent evt)
		{
			// action
			if (evt.getIndex() == model.getBaseservo()) {
				model.setBasepos_current(evt.getValue());
				view.setStatusBasepos(Double.toString(model.getBasepos()), Double.toString(model.getBasepos_current()));
			} else if (evt.getIndex() == model.getArmservo()) {
				model.setArmpos_current(evt.getValue());
				view.setStatusArmpos(Double.toString(model.getArmpos()), Double.toString(model.getArmpos_current()));
			} else if (evt.getIndex() == model.getHeadservo()) {
				model.setHeadpos_current(evt.getValue());
				view.setHeadpos(Double.toString(model.getHeadpos()), Double.toString(model.getHeadpos_current()));
			}
		}
	}


	class connectListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			connect();
		}
	}
	class baseLeftListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			System.out.println("clicked baseLeft");
			baseLeft();
		}
	}
	class baseRightListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			baseRight();
		}
	}
	class armForwardListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			armForward();
		}
	}
	class armBackwardListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			armBackward();
		}
	}
	class headUpListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			headUp();
		}
	}
	class headDownListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			headDown();
		}
	}
	class pumpListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			pump();
		}
	}
	class triggerListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			trigger();
		}
	}

	class engageListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			engage();
		}
	}
	class resetListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// action
			resetServos();
		}
	}
	class quitListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// avslutt programmet
			System.exit(0);
		}
	}
	class wikiListener implements ActionListener{
		public void actionPerformed(ActionEvent evt)
		{
			// show wiki page in web browser
			openWebpage("http://beta.ivancover.com/wiki/index.php/Robotic_Arm");
		}
	}
	class phidgetattachListener implements AttachListener{
		public void attached(AttachEvent se) {
			// pidget connected
			System.out.println("Attached: "+se);
			try {
				if (ifk.isAttached()) {
					resetServos();
				}
			} catch (Exception ex) {
				System.out.println("Error: "+ex.getMessage());
			}
		}
	}
	class phidgetdetachListener implements DetachListener{
		public void detached(DetachEvent se) {
			// pidget disconnected
			System.out.println("Detached: "+se);		
		}
	}
	class serverconnectListener implements ServerConnectListener{
		public void serverConnected(ServerConnectEvent ae){
			// server/api connected
			System.out.println("Connected: "+ae);
			connected();			
		}
	}
	class serverdisconnectListener implements ServerDisconnectListener{
		public void serverDisconnected(ServerDisconnectEvent ae){
			// server/api disconnected
			System.out.println("Disconnected: "+ae);
			disconnected();			
		}
	}
	/*
	class keyListener implements  KeyListener{
		public void keyTyped(KeyEvent evt) {}
		public void keyReleased(KeyEvent evt) {
			System.out.println("key released");
			srcMovement = "";
			timer.stop();			
		}
		public void keyPressed(KeyEvent evt) {
			System.out.println("key press "+evt.getKeyChar());
			Character key = evt.getKeyChar();

			if (key.toString().equals("a")) {
				srcMovement = "baseLeft";
			} else if (key.toString().equals("d")) {						
				srcMovement = "baseRight";
			} else if (key.toString().equals("r")) {						
				srcMovement = "armForward";
			} else if (key.toString().equals("c")) {						
				srcMovement = "armBackward";
			} else if (key.toString().equals("w")) {						
				srcMovement = "headUp";
			} else if (key.toString().equals("s")) {						
				srcMovement = "headDown";
			} else if (key.toString().equals("space")) {						
				srcMovement = "trigger";
			}
			robotMovement();
		}
	}
	 */


	class keyListener implements  KeyListener{

		public void keyTyped(KeyEvent evt) {}
		public void keyReleased(KeyEvent evt) {
			System.out.println("key released");
			Character key = evt.getKeyChar();

			if (key.toString().equals("f")) {
				System.out.println("key released, stop firing.. ");
				trigger();
				trigger_not_active = true;
			} 
		}
		public void keyPressed(KeyEvent evt) {
			System.out.println("key press "+evt.getKeyChar());
			Character key = evt.getKeyChar();

			if (key.toString().equals("f") && trigger_not_active) {
				System.out.println("key for firing used, firing.. ");
				trigger_not_active = false;
				trigger();
			} 
		}
	}
	
	private class mouseListener implements MouseListener {
		String srcMovement;
		public mouseListener(String string) {
			srcMovement = string;
		}
		public void mouseMoved(MouseEvent ev) {}
		public void mouseClicked(MouseEvent ev) {}
		public void mouseEntered(MouseEvent ev) {}
		public void mouseExited(MouseEvent ev) {}

		public void mouseReleased(MouseEvent ev){
			System.out.println("mouse released");
			System.out.println("command: "+srcMovement);

			if (srcMovement == "trigger") {						
				System.out.println("stop firing...");
				trigger();
				trigger_not_active = true;
			} else {
				timer.stop();				
			}
		}
		public void mousePressed(MouseEvent ev){
			System.out.println("mouse press");
			System.out.println("command: "+srcMovement);

			if (srcMovement == "trigger" && trigger_not_active) {				
				System.out.println("firing...");
				trigger_not_active = false;
				trigger();
			} else {

				int timing;
				if (srcMovement == "baseLeft" || srcMovement == "baseRight") {
					timing = 15; //25;
				} else if (srcMovement == "headUp" || srcMovement == "headDown") {
					timing = 30;
				} else {
					timing = 50;
				}

				timer = new Timer(timing, new AbstractAction() {
					public void actionPerformed(ActionEvent e) {
						if (srcMovement == "baseLeft") {
							baseLeft();
						} else if (srcMovement == "baseRight") {						
							baseRight();
						} else if (srcMovement == "armForward") {						
							armForward();
						} else if (srcMovement == "armBackward") {						
							armBackward();
						} else if (srcMovement == "headUp") {						
							headUp();
						} else if (srcMovement == "headDown") {						
							headDown();
						} 
					}
				});
				timer.start();
			}
		}
	}
}
