import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.KeyListener;
import java.awt.event.MouseListener;
import java.math.BigDecimal;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.border.Border;




public class View extends JFrame {
	ClassLoader images;

	private JPanel p_status;
	
	private JLabel connected_txt;
	private JLabel connected;
	private JLabel basepos_txt;
	private JLabel basepos;
	private JLabel basepos_current;
	private JLabel armpos_txt;
	private JLabel armpos;
	private JLabel armpos_current;
	private JLabel headpos_txt;
	private JLabel headpos;
	private JLabel headpos_current;
	private JLabel pump_txt;
	private JLabel pump;
	private JLabel trigger_txt;
	private JLabel trigger;

	private JPanel p_conn_cont_container;
	private JPanel p_connect;
	private JLabel address_txt;
	private JTextField jt_address;
	private JButton jb_connect;

	private JPanel p_controls;
	private JPanel p_controls_inner;
	private JButton jb_reset;
	private JButton jb_arm_forward;
	private JButton jb_arm_backward;
	private JButton jb_head_up;
	private JButton jb_head_down;
	private JButton jb_base_left;
	private JButton jb_base_right;
	private JButton jb_trigger;
	private JButton jb_engage;
	private JButton jb_pump;

	private JMenu jm1;
	private JMenuItem jm1_connect;
	private JMenuItem jm1_quit;

	private JMenu jm2;
	private JMenuItem jm2_cut;
	private JMenuItem jm2_copy;
	private JMenuItem jm2_paste;

	private JMenu jm3;
	private JMenuItem jm3_reset;
	private JMenuItem jm3_engage;
	private JMenuItem jm3_baseleft;
	private JMenuItem jm3_baseright;
	private JMenuItem jm3_headup;
	private JMenuItem jm3_headdown;
	private JMenuItem jm3_armforward;
	private JMenuItem jm3_armbackward;
	private JMenuItem jm3_pump;
	private JMenuItem jm3_trigger;

	private JMenu jm4;
	private JMenuItem jm4_wiki;

	public View(){

		// set system default look and feel
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception unused) {
			// ignore exception because we can't do anything.  will use default.
		}

		// menu
		addMenu();

		// gui components
		buildComponents();

		// frame settings
		this.setTitle("WCR: Water Cannon Robot");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setSize(700,300);
		//this.pack();
		this.setLocation(150,150);
		this.setResizable(false);
		this.setFocusable(true);
		this.getRootPane().setDefaultButton(jb_connect);

		this.setVisible(true);
	}

	/**
	 * GridBag can be hard to setup, this metode is a simple way to set the positions and dimensions of a cell 
	 * before adding an element to it.
	 * 
	 * @param gbc GridBag reference to make the change.
	 * @param row the row to add the element
	 * @param column the column to add the element
	 * @param width the cell width of the element
	 * @param height the cell height of the element
	 * @param fill how the cell should behave
	 */
	private void set_gbc(GridBagConstraints gbc, int row, int column, int width, int height, int fill) {
		gbc.gridy = row;
		gbc.gridx = column;
		gbc.gridwidth = width;
		gbc.gridheight = height;
		gbc.fill = fill;  // GridBagConstraints.NONE .HORIZONTAL .VERTICAL .BOTH
		// leave other fields (eg, anchor) unchanged.
	}

	/**
	 * Setup and build the GUI.
	 * 
	 */
	private void buildComponents() {
		Container cp=this.getContentPane();
		GridBagConstraints gbc = new GridBagConstraints();

		// status section
		p_status = new JPanel();
		Border p_status_border = BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(10,10,10,10), BorderFactory.createTitledBorder(" Status "));
		p_status.setBorder(p_status_border);

		gbc.insets = new Insets(5,10,15,50);

		//p_status.setLayout(new GridLayout(2,2,10,10));
		p_status.setLayout(new GridBagLayout());

		connected_txt = new JLabel("Connected:");
		this.set_gbc(gbc,0, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(connected_txt,gbc);
		connected = new JLabel("false");
		this.set_gbc(gbc,0, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(connected,gbc);
		basepos_txt = new JLabel("Base position:");
		this.set_gbc(gbc,1, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(basepos_txt,gbc);
		basepos = new JLabel("0");
		this.set_gbc(gbc,1, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(basepos,gbc);
		armpos_txt = new JLabel("Arm position:");
		this.set_gbc(gbc,2, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(armpos_txt,gbc);
		armpos = new JLabel("0");
		this.set_gbc(gbc,2, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(armpos,gbc);
		headpos_txt = new JLabel("Head position:");
		this.set_gbc(gbc,3, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(headpos_txt,gbc);
		headpos = new JLabel("0");
		this.set_gbc(gbc,3, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(headpos,gbc);
		pump_txt = new JLabel("Pump status:");
		this.set_gbc(gbc,4, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(pump_txt,gbc);
		pump = new JLabel("false");
		this.set_gbc(gbc,4, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(pump,gbc);
		trigger_txt = new JLabel("Trigger firing:");
		this.set_gbc(gbc,5, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(trigger_txt,gbc);
		trigger = new JLabel("false");
		this.set_gbc(gbc,5, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		p_status.add(trigger,gbc);

		// connect section
		p_connect = new JPanel();
		Border p_connect_border = BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0,0,0,0), BorderFactory.createTitledBorder(" Connect "));
		p_connect.setBorder(p_connect_border);
		p_connect.setLayout(new GridBagLayout());
		p_connect.setPreferredSize(new Dimension(430,58));

		gbc.insets = new Insets(9,10,15,26);
		
		address_txt = new JLabel("Server address:");
		this.set_gbc(gbc,0, 0, 1, 1, GridBagConstraints.HORIZONTAL);
		p_connect.add(address_txt,gbc);

		jt_address = new JTextField("localhost", 100);
		this.set_gbc(gbc,0, 1, 1, 1, GridBagConstraints.HORIZONTAL);
		jt_address.setMinimumSize(new Dimension(140,23));
		p_connect.add(jt_address,gbc);

		jb_connect = new JButton("  Connect  ");
		this.set_gbc(gbc,0, 2, 1, 1, GridBagConstraints.HORIZONTAL);
		p_connect.add(jb_connect,gbc);
		
		// control section
		p_controls = new JPanel();
		Border p_controls_border = BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0,0,0,0), BorderFactory.createTitledBorder(" Controls "));
		p_controls.setBorder(p_controls_border);
		p_controls.setLayout(new GridLayout(1,1,1,1));
		p_controls.setPreferredSize(new Dimension(430,170));

		gbc.insets = new Insets(0,0,1,1);  // put spacing around this field.

		p_controls_inner = new JPanel();
		Border p_controls_inner_border = BorderFactory.createEmptyBorder(10,10,10,10);
		p_controls_inner.setBorder(p_controls_inner_border);

		p_controls_inner.setLayout(new GridLayout(3,5,1,1));

		jb_reset = new JButton("Reset");
		this.set_gbc(gbc,0, 0, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_reset,gbc);

		p_controls_inner.add(new JLabel(""));

		jb_head_up = new JButton("Up");
		this.set_gbc(gbc,0, 2, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_head_up,gbc);

		p_controls_inner.add(new JLabel(""));

		jb_arm_forward = new JButton("Forw.");
		this.set_gbc(gbc,0, 4, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_arm_forward,gbc);

		jb_pump = new JButton("Pump On");
		this.set_gbc(gbc,1, 1, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_pump,gbc);

		jb_base_left = new JButton("Left");
		this.set_gbc(gbc,1, 1, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_base_left,gbc);
		jb_trigger = new JButton("Fire");
		this.set_gbc(gbc,1, 2, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_trigger,gbc);
		jb_base_right = new JButton("Right");
		this.set_gbc(gbc,1, 3, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_base_right,gbc);

		p_controls_inner.add(new JLabel(""));

		jb_engage = new JButton("Engage");
		this.set_gbc(gbc,2, 0, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_engage,gbc);
		
		p_controls_inner.add(new JLabel(""));
		jb_head_down = new JButton("Down");
		this.set_gbc(gbc,2, 2, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_head_down,gbc);

		p_controls_inner.add(new JLabel(""));

		jb_arm_backward = new JButton("Backw.");
		this.set_gbc(gbc,2, 4, 1, 1, GridBagConstraints.BOTH);
		p_controls_inner.add(jb_arm_backward,gbc);

		p_controls.add(p_controls_inner);

		// new container for connect and controls
		p_conn_cont_container = new JPanel();
		Border p_conn_cont_container_border = BorderFactory.createEmptyBorder(5,10,10,10);
		p_conn_cont_container.setBorder(p_conn_cont_container_border);
		p_conn_cont_container.add(p_connect, BorderLayout.NORTH);
		p_conn_cont_container.add(p_controls, BorderLayout.SOUTH);
		
		// add sections to container
		cp.add(p_status, BorderLayout.WEST);
		cp.add(p_conn_cont_container, BorderLayout.CENTER);
	}

	/**
	 * Create the menus in the application.
	 */
	private void addMenu(){
		JMenuBar jmb=new JMenuBar();
		this.setJMenuBar(jmb);
		
		jm1=new JMenu("File");
		jmb.add(jm1);
		jm1_connect =new JMenuItem("Connect");
		jm1_connect.setAccelerator(KeyStroke.getKeyStroke("control K"));
		jm1.add(jm1_connect);
		jm1.addSeparator();
		jm1_quit =new JMenuItem("Quit");
		jm1_quit.setAccelerator(KeyStroke.getKeyStroke("control Q"));
		jm1.add(jm1_quit);

		jm2=new JMenu("Edit");
		jmb.add(jm2);
		jm2_cut = new JMenuItem("Cut");
		jm2_cut.setAccelerator(KeyStroke.getKeyStroke("control X"));
		jm2_copy = new JMenuItem("Copy");
		jm2_copy.setAccelerator(KeyStroke.getKeyStroke("control C"));
		jm2_paste = new JMenuItem("Paste");
		jm2_paste.setAccelerator(KeyStroke.getKeyStroke("control V"));
		jm2.add(jm2_cut);
		jm2.add(jm2_copy);
		jm2.add(jm2_paste);

		jm3=new JMenu("Controls");
		jmb.add(jm3);
		jm3_reset = new JMenuItem("Reset");
		jm3_reset.setAccelerator(KeyStroke.getKeyStroke("Q"));
		jm3.add(jm3_reset);
		jm3_engage = new JMenuItem("Engage Arm");
		jm3_engage.setAccelerator(KeyStroke.getKeyStroke("E"));
		jm3.add(jm3_engage);
		jm3.addSeparator();
		jm3_baseleft = new JMenuItem("Base Left");
		jm3_baseleft.setAccelerator(KeyStroke.getKeyStroke("A"));
		jm3.add(jm3_baseleft);
		jm3_baseright = new JMenuItem("Base Right");
		jm3_baseright.setAccelerator(KeyStroke.getKeyStroke("D"));
		jm3.add(jm3_baseright);
		jm3_headup = new JMenuItem("Head Up");
		jm3_headup.setAccelerator(KeyStroke.getKeyStroke("W"));
		jm3.add(jm3_headup);
		jm3_headdown = new JMenuItem("Head Down");
		jm3_headdown.setAccelerator(KeyStroke.getKeyStroke("S"));
		jm3.add(jm3_headdown);
		jm3_armforward = new JMenuItem("Arm Forward");
		jm3_armforward.setAccelerator(KeyStroke.getKeyStroke("R"));
		jm3.add(jm3_armforward);
		jm3_armbackward = new JMenuItem("Arm Backward");
		jm3_armbackward.setAccelerator(KeyStroke.getKeyStroke("C"));
		jm3.add(jm3_armbackward);
		jm3_pump = new JMenuItem("Pump");
		jm3_pump.setAccelerator(KeyStroke.getKeyStroke("P"));
		jm3.add(jm3_pump);
		jm3_trigger = new JMenuItem("Trigger");
		jm3_trigger.setAccelerator(KeyStroke.getKeyStroke("F"));
		jm3.add(jm3_trigger);

		jm4=new JMenu("About");
		jmb.add(jm4);
		jm4_wiki = new JMenuItem("Wiki page");
		jm4.add(jm4_wiki);		
	}

	public String round(String str) {
		BigDecimal bd = new BigDecimal(str);
		bd = bd.setScale(2,BigDecimal.ROUND_HALF_UP);
		double newpos = bd.doubleValue();
		return Double.toString(newpos);
	}

	public String getAddress() {
		return jt_address.getText();
	}

	public void setConnect_button(String str) {
		this.jb_connect.setText(str);
		this.jm1_connect.setText(str);
		if (str.equals("Disconnect")) {
			this.jt_address.setEnabled(false);
		} else {
			this.jt_address.setEnabled(true);
		}
	}
	public void setStatusConnected(String str) {
		this.connected.setText(str);
	}

	public void setStatusConnectedColor(Color clr) {
		this.connected.setForeground(clr);
	}

	public void setStatusBasepos(String basepos, String basepos_current) {
		this.basepos.setText(round(basepos)+"");
	}

	public void setStatusArmpos(String armpos, String armpos_current) {
		this.armpos.setText(round(armpos)+"");
	}

	public void setHeadpos(String headpos, String headpos_current) {
		this.headpos.setText(round(headpos)+"");
	}

	public void setStatusPump(String pump) {
		this.pump.setText(pump);
	}

	public void setStatusTrigger(String trigger) {
		this.trigger.setText(trigger);
	}


	public void setControlsPump(String pump) {
		this.jb_pump.setText(pump);
	}

	public void setControlsEngage(boolean engage) {
		if (engage == false) {
			this.jb_engage.setText("Disable");
			this.jm3_engage.setText("Disable Arm");
		} else {
			this.jb_engage.setText("Engage");			
			this.jm3_engage.setText("Engage Arm");
		}
	}
	 
	// LISTENERS
	/* Controls */
	public void addListenerToConnect(ActionListener al){
		jb_connect.addActionListener(al);
	}	
	public void addListenerToBaseLeft(ActionListener al){
		jb_base_left.addActionListener(al);
	}
	public void addMouseListenerToBaseLeft(MouseListener al){
		jb_base_left.addMouseListener(al);
	}
	public void addListenerToBaseRight(ActionListener al){
		jb_base_right.addActionListener(al);
	}
	public void addMouseListenerToBaseRight(MouseListener al){
		jb_base_right.addMouseListener(al);
	}
	public void addListenerToArmForward(ActionListener al){
		jb_arm_forward.addActionListener(al);
	}
	public void addMouseListenerToArmForward(MouseListener al){
		jb_arm_forward.addMouseListener(al);
	}
	public void addListenerToArmBackward(ActionListener al){
		jb_arm_backward.addActionListener(al);
	}
	public void addMouseListenerToArmBackward(MouseListener al){
		jb_arm_backward.addMouseListener(al);
	}
	public void addListenerToHeadUp(ActionListener al){
		jb_head_up.addActionListener(al);
	}
	public void addMouseListenerToHeadUp(MouseListener al){
		jb_head_up.addMouseListener(al);
	}
	public void addListenerToHeadDown(ActionListener al){
		jb_head_down.addActionListener(al);
	}
	public void addMouseListenerToHeadDown(MouseListener al){
		jb_head_down.addMouseListener(al);
	}
	public void addListenerToPump(ActionListener al){
		jb_pump.addActionListener(al);
	}
	public void addListenerToTrigger(ActionListener al){
		jb_trigger.addActionListener(al);
	}
	public void addMouseListenerToTrigger(MouseListener al){
		jb_trigger.addMouseListener(al);
	}
	public void addListenerToKey(KeyListener al){
		this.addKeyListener(al);
		jt_address.addKeyListener(al);
		jb_connect.addKeyListener(al);
		jb_base_left.addKeyListener(al);
		jb_base_right.addKeyListener(al);
		jb_arm_forward.addKeyListener(al);
		jb_arm_backward.addKeyListener(al);
		jb_head_up.addKeyListener(al);
		jb_head_down.addKeyListener(al);
		jb_pump.addKeyListener(al);
		jb_trigger.addKeyListener(al);
		jb_engage.addKeyListener(al);
		jb_reset.addKeyListener(al);
	}

	public void addListenerToEngage(ActionListener al){
		jb_engage.addActionListener(al);
	}
	public void addListenerToReset(ActionListener al){
		jb_reset.addActionListener(al);
	}
	/* File menu */
	public void addListenerToQuit(ActionListener al){
		jm1_quit.addActionListener(al);
	}
	public void addListenerToConnectTo(ActionListener al){
		jm1_connect.addActionListener(al);
	}
	/* Controls menu */
	public void addListenerToControlsReset(ActionListener al){
		jm3_reset.addActionListener(al);
	}
	public void addListenerToControlsEngage(ActionListener al){
		jm3_engage.addActionListener(al);
	}
	public void addListenerToControlsBaseLeft(ActionListener al){
		jm3_baseleft.addActionListener(al);
	}
	public void addListenerToControlsBaseRight(ActionListener al){
		jm3_baseright.addActionListener(al);
	}
	public void addListenerToControlsHeadUp(ActionListener al){
		jm3_headup.addActionListener(al);
	}
	public void addListenerToControlsHeadDown(ActionListener al){
		jm3_headdown.addActionListener(al);
	}
	public void addListenerToControlsArmForward(ActionListener al){
		jm3_armforward.addActionListener(al);
	}
	public void addListenerToControlsArmBackward(ActionListener al){
		jm3_armbackward.addActionListener(al);
	}
	public void addListenerToControlsPump(ActionListener al){
		jm3_pump.addActionListener(al);
	}
	public void addListenerToControlsTrigger(ActionListener al){
		jm3_trigger.addActionListener(al);
	}
	/* About menu */
	public void addListenerToWiki(ActionListener al){
		jm4_wiki.addActionListener(al);
	}
	/*
	public void addListenerToKey(KeyListener al){
		jb_reset.addKeyListener(al);
	}
	*/
}
